@@ -9,7 +9,7 @@ Usage:
 [--onlymp3][--hide-progress][--min-size <size>][--max-size <size>][--remove][--no-album-tag]
 [--no-playlist-folder][--download-archive <file>][--extract-artist][--flac][--original-art]
 [--original-name][--no-original][--only-original][--name-format <format>][--strict-playlist]
-[--playlist-name-format <format>][--client-id <id>][--auth-token <token>][--overwrite]
+[--playlist-name-format <format>][--client-id <id>][--auth-token <token>][--overwrite][--no-playlist]
     scdl -h | --help
     scdl --version
 
@@ -59,11 +59,13 @@ Options:
     --auth-token [token]            Specify the auth token to use
     --overwrite                     Overwrite file if it already exists
     --strict-playlist               Abort playlist downloading if one track fails to download
+    --no-playlist                   Skip downloading playlists
 """
 
 import cgi
 import configparser
 import logging
+import math
 import mimetypes
 import pathlib
 
@@ -89,7 +91,8 @@ import requests
 from clint.textui import progress
 from docopt import docopt
 from pathvalidate import sanitize_filename
-from soundcloud import BasicAlbumPlaylist, BasicTrack, MiniTrack, SoundCloud
+from soundcloud import (BasicAlbumPlaylist, BasicTrack, MiniTrack, SoundCloud,
+                        Transcoding)
 
 from scdl import __version__, utils
 
@@ -172,7 +175,7 @@ def main():
             python_args["offset"] = int(arguments["-o"])
             if python_args["offset"] < 1:
                 raise ValueError()
-        except:
+        except Exception:
             logger.error("Offset should be a positive integer...")
             sys.exit(1)
         logger.debug("offset: %d", python_args["offset"])
@@ -180,7 +183,7 @@ def main():
     if arguments["--min-size"] is not None:
         try:
             arguments["--min-size"] = utils.size_in_bytes(arguments["--min-size"])
-        except:
+        except Exception:
             logger.exception(
                 "Min size should be an integer with a possible unit suffix"
             )
@@ -190,7 +193,7 @@ def main():
     if arguments["--max-size"] is not None:
         try:
             arguments["--max-size"] = utils.size_in_bytes(arguments["--max-size"])
-        except:
+        except Exception:
             logger.error("Max size should be an integer with a possible unit suffix")
             sys.exit(1)
         logger.debug("max-size: %d", arguments["--max-size"])
@@ -208,7 +211,7 @@ def main():
         # set url to profile associated with auth token
         arguments["-l"] = client.get_me().permalink_url
     
-    arguments["-l"] = validate_url(arguments["-l"])
+    arguments["-l"] = validate_url(client, arguments["-l"])
         
     # convert arguments dict to python_args (kwargs-friendly args)
     for key, value in arguments.items():
@@ -232,7 +235,7 @@ def main():
     if arguments["--remove"]:
         remove_files()
 
-def validate_url(url: str):
+def validate_url(client: SoundCloud, url: str):
     """
     If url is a valid soundcloud.com url, return it.
     Otherwise, try to fix the url so that it is valid.
@@ -249,10 +252,14 @@ def validate_url(url: str):
         return url
     
     # see if link redirects to soundcloud.com
-    resp = requests.get(url)
-    if url.startswith("https://soundcloud.com") or url.startswith("http://soundcloud.com"):
-        url = urllib.parse.urljoin(resp.url, urllib.parse.urlparse(resp.url).path)
-        return url
+    try:
+        resp = requests.get(url)
+        if url.startswith("https://soundcloud.com") or url.startswith("http://soundcloud.com"):
+            return urllib.parse.urljoin(resp.url, urllib.parse.urlparse(resp.url).path)
+    except Exception:
+        # see if given a username instead of url
+        if client.resolve(f"https://soundcloud.com/{url}"):
+            return f"https://soundcloud.com/{url}"
     
     logger.error("URL is not valid")
     sys.exit(1)
@@ -384,6 +391,9 @@ def download_playlist(client: SoundCloud, playlist: BasicAlbumPlaylist, **kwargs
     """
     Downloads a playlist
     """
+    if kwargs.get("no_playlist"):
+        logger.info("Skipping playlist...")
+        return
     playlist_name = playlist.title.encode("utf-8", "ignore")
     playlist_name = playlist_name.decode("utf-8")
     playlist_name = sanitize_filename(playlist_name)
@@ -420,7 +430,7 @@ def download_playlist(client: SoundCloud, playlist: BasicAlbumPlaylist, **kwargs
 def try_utime(path, filetime):
     try:
         os.utime(path, (time.time(), filetime))
-    except:
+    except Exception:
         logger.error("Cannot update utime of file")
 
 def get_filename(track: BasicTrack, original_filename=None, aac=False, playlist_info=None, **kwargs):
@@ -499,6 +509,13 @@ def download_original_file(client: SoundCloud, track: BasicTrack, title: str, pl
 
     # Write file
     total_length = int(r.headers.get("content-length"))
+    
+    min_size = kwargs.get("min_size") or 0
+    max_size = kwargs.get("max_size") or math.inf # max size of 0 treated as no max size
+    
+    if not min_size <= total_length <= max_size:
+        raise SoundCloudException("File not within --min-size and --max-size bounds")
+    
     temp = tempfile.NamedTemporaryFile(delete=False)
     received = 0
     with temp as f:
@@ -530,14 +547,16 @@ def download_original_file(client: SoundCloud, track: BasicTrack, title: str, pl
     return (filename, False)
 
 
-def get_track_m3u8(client: SoundCloud, track: BasicTrack, aac=False):
-    url = None
-    for transcoding in track.media.transcodings:
-        if transcoding.format.protocol == "hls":
-            if (not aac and transcoding.format.mime_type == "audio/mpeg") or (
-                aac and transcoding.format.mime_type.startswith("audio/mp4")
-            ):
-                url = transcoding.url
+def get_track_m3u8(client: SoundCloud, track: BasicTrack, transcoding: Transcoding, **kwargs):
+    url = transcoding.url
+    bitrate_KBps = 256 / 8 if transcoding.preset == "aac_hq" else 128 / 8
+    total_bytes = bitrate_KBps * transcoding.duration
+    
+    min_size = kwargs.get("min_size") or 0
+    max_size = kwargs.get("max_size") or math.inf # max size of 0 treated as no max size
+    
+    if not min_size <= total_bytes <= max_size:
+        raise SoundCloudException("File not within --min-size and --max-size bounds")
 
     if url is not None:
         headers = client.get_default_headers()
@@ -552,14 +571,20 @@ def download_hls(client: SoundCloud, track: BasicTrack, title: str, playlist_inf
 
     if not track.media.transcodings:
         raise SoundCloudException(f"Track {track.permalink_url} has no transcodings available")
-
-    if kwargs["onlymp3"]:
-        aac = False
-    else:
-        aac = any(
-            t.format.mime_type.startswith("audio/mp4")
-            for t in track.media.transcodings
-        )
+    
+    logger.debug(f"Trancodings: {track.media.transcodings}")
+    
+    transcodings = {t.preset: t for t in track.media.transcodings if t.format.protocol == "hls"}
+    transcoding = None
+    aac = False
+    if not kwargs.get("onlymp3") and "aac_hq" in transcodings:
+        transcoding = transcodings["aac_hq"]
+        aac = True
+    elif "mp3_0_0" in transcodings:
+        transcoding = transcodings["mp3_0_0"]
+                
+    if not transcoding:
+        raise SoundCloudException(f"Could not find mp3_0_0 or aac_hq transcoding. Available transcodings: {list(transcodings)}")
 
     filename = get_filename(track, None, aac, playlist_info, **kwargs)
     logger.debug(f"filename : {filename}")
@@ -568,7 +593,7 @@ def download_hls(client: SoundCloud, track: BasicTrack, title: str, playlist_inf
         return (filename, True)
 
     # Get the requests stream
-    url = get_track_m3u8(client, track, aac)
+    url = get_track_m3u8(client, track, transcoding, **kwargs)
     filename_path = os.path.abspath(filename)
 
     p = subprocess.Popen(
@@ -636,7 +661,7 @@ def download_track(client: SoundCloud, track: BasicTrack, playlist_info=None, ex
         ):
             try:
                 set_metadata(track, filename, playlist_info, **kwargs)
-            except:
+            except Exception:
                 os.remove(filename)
                 logger.exception("Error trying to set the tags...")
                 raise SoundCloudException("Error trying to set the tags...")
@@ -753,7 +778,7 @@ def set_metadata(track: BasicTrack, filename: str, playlist_info=None, **kwargs)
                 "image/jpg",
             ):
                 response = None
-        except:
+        except Exception:
             pass
     if response is None:
         new_artwork_url = artwork_url.replace("large", "t500x500")
